<?php
//$Id:
include_once('Istorage.php');

/**
 * This function needs to be implemented!!!
 * @return Istorage
 */
function fast_gallery_get_storage() {
  return new DefaultStorage();
}

class DefaultStorage implements Istorage {
  /**
   * Return all images that are stored on a given path
   *
   * @param $path String
   * @return array - an array of FGImages
   */
  public function getImages($path) {
    //incase we have a / at the end -> remove it.
    $arTmp = explode("/", $path);
    if ($arTmp[sizeof($arTmp)-1] == '') {
      array_pop($arTmp);
    }
    $folder = implode("/", $arTmp);
    $return = array();

    //incase we want to display the subfolder first
    if (variable_get('fg_folder_first', FALSE)) {
      $foid = db_query("SELECT foid FROM {fast_gallery_hierarchy} WHERE folder=:folder", array(':folder' => $folder))->fetchField();
      $return = array_merge($return, $this->getFolders($foid));
    }

    //fetch images
    $row = db_query("SELECT filename, folder, fgf.foid FROM {fast_gallery_file} as fgf, {fast_gallery_hierarchy} as fgh
                     WHERE fgf.foid = fgh.foid 
                     AND fgh.folder = :folder",
                     array(':folder' => $folder), array('fetch' => PDO::FETCH_ASSOC));

    $foid = 0;
    foreach ($row as $item) {
      $return[] = new FGImage($item['folder'] . '/' . $item['filename']);
      $foid = $item['foid'];
    }

    //incase we display the subfolders last
    if (!variable_get('fg_folder_first', FALSE)) {
      $return = array_merge($return, $this->getFolders($foid));
    }

    return $return;
  }

  /**
   * Get the subfolders of a given folder
   * @param int $foid
   * @return array
   */
  private function getFolders($foid) {
    $folders = array();
    $row = db_query("SELECT folder FROM {fast_gallery_hierarchy} WHERE pid = :foid", array(':foid' => $foid), array('fetch' => PDO::FETCH_ASSOC));
    foreach ($row as $folder) {
      $folders[] = new FGImage($folder['folder'], array('dir' => TRUE));
    }
    return $folders;
  }

  /**
   * implementing the interface. Passing an array of
   * FGImages and save them into the db
   * @param array $arImages
   *  an array of FGImages
   *
   */
  public function storeImages($arImages) {
    global $user;
    static $first_run = TRUE;
    static $runid;
    $runid = variable_get('fast_gallery_runid', 0);

    //TODO: This makes the APP slow -> Rebuilding the whole
    // Gallery every time!
    if ($first_run) {
      db_query("TRUNCATE TABLE  `fast_gallery_hierarchy`");
      $first_run = FALSE;
      variable_set('fast_gallery_runid', ++$runid);
    }

    foreach ($arImages as $image) {
      $foid = $this->storeHierarchy($image);

      db_merge('fast_gallery_file')
      ->key(array('foid' => $foid, 'filename' => $image->getName()))
      ->fields(array('runid' => $runid))
      ->execute();
    }
  }

  /**
   * (non-PHPdoc)
   * @see sites/all/modules/fast_gallery/storageengine/Istorage#clearDb()
   */
  public function clearDb() {
    db_truncate('fast_gallery_hierarchy')->execute();
    db_truncate('fast_gallery_file')->execute();
  }

  /**
   * (non-PHPdoc)
   * @see sites/all/modules/fast_gallery/storageengine/Istorage#removeDeletedFiles()
   */
  public function removeDeletedFiles() {
    db_query("DELETE FROM {fast_gallery_file} WHERE runid < :runid", array('runid' => variable_get('fast_gallery_runid', 0)));
  }

  /**
   * In order to retrieve the hierarchy we need to store this information
   * in a special table
   * @param FGImage $image
   * @return unknown_type
   */
  private function storeHierarchy($image) {
    static $folders = array();

    //a little cache to reduce load on db -> we only have to run this for one image per folder
    if (!in_array($image->getFolder(), $folders)) {
      $folder = db_query("SELECT folder AS c FROM {fast_gallery_hierarchy}
              WHERE folder = :folder", array(':folder' => $image->getFolder()))->fetchField();

      if (!$folder) { //incase we don't have this folder in our db
        $dbObject['folder'] = $image->getFolder();
        $dbObject['pid'] = db_query("SELECT foid FROM {fast_gallery_hierarchy}
                            WHERE folder = :folder", array(':folder' => $image->getParent()))->fetchField();
        drupal_write_record('fast_gallery_hierarchy', $dbObject);

        //TODO: needs to be checked!!! needs to return the foid, so that we can make
        //the correct entry into fast_gallery
        $foid = db_query("SELECT MAX(foid) FROM {fast_gallery_hierarchy}")->fetchField();
        $folders[$foid] = $image->getFolder();
        return $foid;
      }
    }
    $ar = array_flip($folders);
    return $ar[$image->getFolder()];
  }



}